"""Bundled Chart.js distribution for offline use."""

CHARTJS_VERSION = "4.4.3"
SCRIPT_NAME = "chart.umd.min.js"
CHARTJS_FILENAME = SCRIPT_NAME
LICENSE_NAME = "LICENSE"

__all__ = [
    "CHARTJS_VERSION",
    "SCRIPT_NAME",
    "CHARTJS_FILENAME",
    "LICENSE_NAME",
]
