.cards {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(220px, 1fr));
  gap: 1.25rem;
}

.card {
  border: 1px solid var(--wd-border-muted);
  border-radius: 0.75rem;
  padding: 1rem;
  background: var(--wd-color-surface);
  box-shadow: var(--wd-shadow-soft);
  display: flex;
  flex-direction: column;
  gap: 0.75rem;
}

.card img {
  width: 100%;
  height: 160px;
  object-fit: cover;
  border-radius: 0.5rem;
}

.card h3 {
  margin: 0;
  font-size: 1.1rem;
  color: var(--wd-text-primary);
}

.card ul {
  margin: 0;
  padding: 0;
  list-style: none;
  display: flex;
  flex-direction: column;
  gap: 0.35rem;
  color: var(--wd-text-secondary);
  font-size: 0.9rem;
}

.card li {
  display: flex;
  gap: 0.35rem;
  align-items: flex-start;
}

.card li span {
  font-weight: 600;
  color: var(--wd-text-primary);
}
