.params-bar {
  margin-bottom: 1.25rem;
  padding: 0.85rem 1rem;
  border: 1px solid var(--wd-border-muted);
  border-radius: 0.75rem;
  background: var(--wd-panel-muted);
  color: var(--wd-text-secondary);
}

.params-form {
  display: flex;
  flex-wrap: wrap;
  gap: 0.75rem;
  align-items: flex-end;
}

.param-field {
  display: flex;
  flex-direction: column;
  gap: 0.35rem;
  min-width: 12rem;
  position: relative;
}

.param-field label {
  font-size: 0.85rem;
  font-weight: 600;
  color: var(--wd-text-secondary);
}

.param-field input,
.param-field select {
  padding: 0.45rem 0.6rem;
  border: 1px solid var(--wd-control-border);
  border-radius: 0.375rem;
  font: inherit;
  background: var(--wd-control-bg);
  color: var(--wd-control-text);
  min-height: 2.25rem;
}

.param-field select {
  min-width: 10rem;
}

.param-field input::placeholder,
.param-field select::placeholder {
  color: var(--wd-control-placeholder);
}

.param-field input:focus-visible,
.param-field select:focus-visible {
  border-color: var(--wd-control-border-strong);
  outline: 2px solid var(--wd-accent-soft);
  outline-offset: 1px;
}

.param-help {
  font-size: 0.75rem;
  color: var(--wd-text-muted);
  margin: 0;
}

.param-actions {
  display: flex;
  align-items: center;
  gap: 0.75rem;
}

.param-actions button {
  padding: 0.45rem 0.95rem;
  border-radius: 0.375rem;
  border: 1px solid var(--wd-accent);
  background: var(--wd-accent);
  color: var(--wd-accent-contrast);
  font: inherit;
  cursor: pointer;
}

.param-actions button:hover {
  background: color-mix(in srgb, var(--wd-accent) 88%, black 12%);
  border-color: color-mix(in srgb, var(--wd-accent) 88%, black 12%);
}

.param-actions .reset-link {
  color: var(--wd-accent);
  text-decoration: none;
  font-size: 0.9rem;
}

.param-actions .reset-link:hover {
  text-decoration: underline;
}

.result-summary {
  margin: 0.5rem 0 1rem;
  color: var(--wd-text-secondary);
  font-size: 0.9rem;
}

.rpc-actions {
  margin-top: 1rem;
  display: flex;
  gap: 1rem;
  align-items: center;
}

.rpc-actions a {
  color: var(--wd-accent);
  text-decoration: none;
  font-weight: 600;
}

.rpc-actions a:hover {
  text-decoration: underline;
}

.pagination {
  margin-top: 1rem;
}

.pagination a {
  color: var(--wd-accent);
  text-decoration: none;
  font-weight: 600;
}

.pagination a:hover {
  text-decoration: underline;
}

@media (max-width: 720px) {
  .params-form {
    flex-direction: column;
    align-items: stretch;
  }

  .param-field {
    width: 100%;
  }
}
